function[inds] = crdt_cross_ratio_indices(z)
% crdt_cross_ratio_indices -- Computes cross-ratio index variables 
%
% inds = crdt_cross_ratio_indices(z)
%
%     Given the length-N complex-valued vector z that contains ordered vertices
%     for a shape, this function returns the size (N-3) x 4 matrix inds, where
%     each row contains 4 indices corresponding to points on the shape to
%     compute a cross-ratio for. Indices are integers in the range 1,..,N.
%
%     The N-3 cross ratios are chose as in the "CRDT" algorithm of Driscoll and
%     Vavasis (1998) -- given the Delaunay triangulation, they are the N-3
%     unique quadrilaterals formed from two adjacent polygon-interior triangles
%     whose diagonal is formed by a non-boundary interior edge of the
%     triangulation. 

% Constrain edges of polygon.
N = length(z);
c = [(1:N)' [(2:N)'; 1]];

tri = DelaunayTri(real(z(:)), imag(z(:)), c);

interior_triangles = find(tri.inOutStatus());

% Now identify the interior edges and trace them back to the triangles
edges = zeros([2*(N-3) 2]);
triangle_indices = zeros([2*(N-3) 1]);
count = 1;
for q = interior_triangles(:)';
  current_triangle = sort(tri.Triangulation(q,:));
  index_set = [current_triangle current_triangle(1)];
  index_diff = diff(index_set);
  % Make last diff the value of 1 if there's a wraparound
  if index_diff(3) == -(N-1);
    index_diff(3) = 1;
  end

  % Label any two non-sequential vertices as an interior edge
  for qq = 1:3;
    if index_diff(qq)~=1;
      edges(count,:) = [index_set(qq), index_set(index_mod(qq+1,3))];
      triangle_indices(count) = q;
      count = count+1;
    end
  end
end

% Now let's sort edges to make identification of a shared edge easier
edges = sort(edges, 2); % monotonicity of columns
temp = sortrows([edges triangle_indices], [1 2]);
edges = temp(:,1:2); 
triangle_indices = temp(:,3);

% There should be N-3 vanishing elements of the following vector, but this is not checked
diffs = sum(abs(diff(edges, [], 1)), 2);
diff_indices = find(diffs==0);

%  pair of triangles to identify as forming a quadrilateral
quad_triangles = [triangle_indices(diff_indices) ...
                  triangle_indices(diff_indices+1)];

% Now just pluck out unique indices and sort for each quad:
all_indices = [tri.Triangulation(quad_triangles(:,1), :) ...
               tri.Triangulation(quad_triangles(:,2), :)];
inds = zeros([length(diff_indices) 4]);
for q = 1:size(inds,1);
  inds(q,:) = unique(all_indices(q,:));
end
