function[image_information] = crdt_precomputations(z, level)
% crdt_precomputations -- Delaunay triangulation + multiscale computations
%
% image_information = crdt_precomputations(z, level)
%
%     Given a length-N vector z along with a non-negative integer 'level',
%     computes the multiscale decomposition (dyadically) along with the
%     requisite CRDT cross-ratios. This information is stored in the struct
%     image_information, which is used by the shooting driver.

N = length(z);

if nargin < 2
  level = 3;
end
if level==0
  index_sets = {1:N};
  stage_additions{1} = 1;
else
  factor = 2^level;

  Nmax = factor*ceil(N/factor);
  index_sets = cell([factor 1]);
  for q = 1:factor
    index_sets{q} = (q-1) + 1:factor:Nmax;
    % Weed out indicies we don't have
    index_sets{q}(index_sets{q}>N) = [];
  end

  for q = 1:(level+1);
    if q==1
      stage_additions{1} = 1;
    elseif q==2
      stage_additions{2} = (factor/2) + 1;
    else
      stage_additions{q} = zeros([1 2^(q-2)]);
      separation = 2^(level-q+2);

      start = 1 + 2^(level+1-q);
      stage_additions{q} = start + (0:(2^(q-2)-1))*separation;
    end

    % Example when level=3:
    % stage_additions = {1, 5, [3 7], [2 4 6 8]};
  end
end

% And now compute the required cross-ratios for each level.
z_points = [];
prev_inds = [];
for stage = 1:length(stage_additions)
  all_new_inds = [];
  for q = 1:length(stage_additions{stage})
    all_new_inds = [all_new_inds; index_sets{stage_additions{stage}(q)}(:)];
  end
  z_points = [z_points; z(all_new_inds)];

  [prev_inds, ordering] = sort([prev_inds; all_new_inds]);
  z_points = z_points(ordering);

  cr_indices{stage} = crdt_cross_ratio_indices(z_points);
end

% Output
image_information.stage_additions = stage_additions;
image_information.index_sets = index_sets;
image_information.cr_indices = cr_indices;
