function[varargout] = objective(teichon_locations, teichon_strengths, particles, particle_goals, cr_indices, fc, dt, lambda)
% objective -- The cross-ratio epdiff objective function
%
% [L] = objective(teichon_locations, teichon_strengths, particles, particle_goals, cr_indices, fc, dt, [[lambda=0]])
% [L,J] = objective(teichon_locations, teichon_strengths, particles, particle_goals, cr_indices, fc, dt, [[lambda=0]])
%
%      Computes the cross-ratio objective for shooting with epdiff. The pair
%      (teichon_locations, teichon_strengths) are used to push the particles to
%      t=1 using a timestep of dt. The locations of these are compared against
%      particle_goals. The comparison is done with cross-ratios: cr_indices is
%      an N x 4 matrix of indices for particles. Each row of
%      particles(cr_indices) has a cross-ratio and the difference between this
%      number each row of particle_goals(cr_indices) is stored. The function
%      handle fc is then to compute the norm of this vector, and this is the
%      objective function.
%
%      If the second calling syntax is used, then the Jacobian of the objective
%      with respect to the teichon_strengths is also computed.
%
%      The final, optional input lambda is a proportionality constant
%      (non-negative) that determines how to add the energy E to the objective.
%      The objective is given by
%
%        L = cross_ratio_norm + lambda*wp_energy,
% 
%      where the wp energy is computed from the initial teichon_locations,
%      teichon_strengths. If lambda is zero, the energy term is ignored.

% Algorithm for Jacobian: chain rule:
%
%  d L     d L     d C     d z     d b
% ----- = ----- x ----- x ----- x -----
% d a_0    d C     d z     d b    d a_0
%
%    L  : objective
%    a_0: initial teichon strengths
%    C  : cross-ratios
%    b  : terminal teichon locations
%    z  : exp(i*b) (teichon locations on disc)

persistent cr 
if isempty(cr)
  cr = CrossRatio();
end

if nargin < 8;
  lambda = 0;
end

compute_jacobian = (nargout > 1);

% Evolve particles
if compute_jacobian
  [a, b, particles, dbda0] = epdiff_teichon_variation(teichon_strengths, teichon_locations, particles, 1, 'dt', dt);
  N = length(teichon_strengths);
  N_particles = length(particles);
  dbda0 = dbda0(2*N+1:end,1:N);
else
  [a, b, particles] = epdiff_teichon(teichon_strengths, teichon_locations, particles, 1, 'dt', dt);
end

% [0, 2*pi] ----> unit circle
particles = exp(i*particles);
particle_goals = exp(i*particle_goals);

if compute_jacobian
  %dzdb = i*spdiag(particles);
  dzda0 = i*spdiag(particles)*dbda0;
end

% Compute (real-valued) cross-ratio of real endpoints
cr_goal = real(cr(particle_goals(cr_indices(:,1)),...
               particle_goals(cr_indices(:,2)),...
               particle_goals(cr_indices(:,3)),... 
               particle_goals(cr_indices(:,4))));

% Compute (real-valued) cross-ratio of current endpoints
cr_comp = real(cr(particles(cr_indices(:,1)),...
               particles(cr_indices(:,2)),...
               particles(cr_indices(:,3)),... 
               particles(cr_indices(:,4))));

if compute_jacobian
  dcdz = cr.jacobian(particles(cr_indices(:,1)),...
                     particles(cr_indices(:,2)),...
                     particles(cr_indices(:,3)),... 
                     particles(cr_indices(:,4)));

  % Second, faster method:
  N_cr = size(cr_indices,1);
  dcda0 = zeros([N_cr N]);
  for q = 1:N_cr
    dcda0(q,:) = dcdz(q,:)*dzda0(cr_indices(q,:),:);
  end

  % Have to 'undo' the indexing done by cr_indices
  %temp = zeros([N_cr, N_particles]);
  %temp2 = repmat((1:N_cr).', [1 4]);
  %temp(sub2ind(size(temp), temp2, cr_indices)) = dcdz;
  %dcdz = temp;
end

% Compute norm of relative error: this is L

if compute_jacobian
  [varargout{1}, dLdc] = fc((cr_comp-cr_goal)./cr_goal, 1);
  dLdc = dLdc./cr_goal(:).';

  varargout{2} = real(dLdc*dcda0);
  %varargout{2} = real(dLdc*dcdz*dzdb*dbda0);
  
else
  varargout{1} = fc((cr_comp-cr_goal)./cr_goal, 0);
end
% Check: secant_jacobian checks out.

if lambda > 0
  G = greens_matrix(teichon_locations);
  varargout{1} = varargout{1} + lambda*teichon_strengths'*G*teichon_strengths;
  if compute_jacobian
    varargout{2} = varargout{2} + 2*lambda*teichon_strengths'*G;
  end
end

if nargout == 3
  % This is the Hessian at this point that computes the quadratic objective,
  % assuming fc(x) = x^2
  %temp = real(dcdz*dzdb*dbda0);
  temp = real(dcda0);
  varargout{3} = temp'*spdiag(1./cr_goal.^2)*temp;
end
