% Performs shooting from the identity to ellipses of various aspect ratios
%
% This is an optimization routine using gradient descent/CG where the objective
% is some norm of a vector of cross-ratio differences at the terminal step.  The
% initial strengths of the teichons are varied in order to minimize this
% objective.  The variation for each piece (epdiff solver, cross-ratio
% computation, etc) is computed exactly.
%
% This is code meant to accompany the paper
% "Approximating the Weil-Petersson Metric Geodesics on the Universal
% Teichmüller space by Singular Solutions", 
% by S. Kushnarev, A. Narayan

clear
close all

aspect_ratio = 2.5;

N_shape = 100;
level = 4;

% Otherwise, you'll have to do something like:
z = ellipse(100, 'a', 1, 'b', 1/aspect_ratio);
map_1 = GeodesicZipperWeld(z);
% ----------------------------------------------------------

% This is cross-ratio information 
cr_scale_information = crdt_precomputations(z, level);

% Data for the identity:
data.tin = linspace(0, 2*pi, length(map_1.exterior_vertices)+1); data.tin(end) = [];
data.tout = data.tin;
% The `origin' map
map_0 = GeodesicZipperWeld([], data.tin, data.tout);

% Let N be the number of total points given in the data set:
N = length(data.tin);

% Extract multiscale information
index_sets = cr_scale_information.index_sets;
stage_additions = cr_scale_information.stage_additions;
all_cr_indices = cr_scale_information.cr_indices;

N_teichons = 100;

% Sample teichons
teichon_locations = linspace(0, 2*pi, N_teichons+1).';
teichon_locations(end) = [];
teichon_strengths = 0*teichon_locations;

% Epdiff parameters:
dt = 0.05;

% Optimization parameters:
N_gradient_descent_iterations = 200;
N_cg_iterations = 10;

% Choose objective
fc = @(vector, d) cross_ratio_norm(vector, d, 2);

cr_indices = zeros([0 4]);
theta_ext = [];
theta_0 = [];
theta_1 = [];
z_points = [];

% "stage": dyadic refinement stages for points on shape
for stage = 1:length(stage_additions);

  % Update theta_0, theta_1
  N_before = length(theta_1);
  all_new_inds = [];
  old_inds = (1:length(theta_0))';
  for q = 1:length(stage_additions{stage})
    new_inds = index_sets{stage_additions{stage}(q)}(:);
    all_new_inds = [all_new_inds; new_inds];

    % ----------------------------------------------------------
    % How do you want to generate points on the fingerprints?

    % This method flows to the exact points given on the map_1 fingerprint.
    % We must interpolate the map_0 fingerprint to make this happen.
    % If you change this, it will break the method used to choose the
    % cross-ratios and you'll have to recode that.
    theta_ext = [theta_ext; map_1.exterior_vertices(new_inds)];
    theta_0 = [theta_0; map_0.interpolate([], map_1.exterior_vertices(new_inds))];
    theta_1 = [theta_1; map_1.interior_vertices(new_inds)];
    z_points = [z_points; map_1.z(new_inds)];
    % ----------------------------------------------------------

  end

  fprintf('Starting matching stage %d, number of cross-ratio points is %d\n', stage, length(theta_0));

  % Sort for cross-ratio determination
  [garbage, sort_order] = sort([old_inds; all_new_inds]);
  
  theta_0 = theta_0(sort_order);
  theta_1 = theta_1(sort_order);
  z_points = z_points(sort_order);

  % "Invert" sort_order
  new_sort_order = zeros([max(cr_indices(:)) 1]);
  for q = 1:length(new_sort_order)
    new_sort_order(q) = find(sort_order==q);
  end

  % Use new ordeing to reindex cr_indices
  cr_indices = new_sort_order(cr_indices);

  % Choose new cross ratios
  cr_indices = [cr_indices; all_cr_indices{stage}];

  %-----------------------------------------------------------------
  % How many teichons do you want to add for each dyadic level?
  N_teichon_additions = 0;
  %-----------------------------------------------------------------

  % Set up observation matrices:
  objective_value = zeros([N_teichon_additions+1 N_gradient_descent_iterations+N_cg_iterations+1]);
  gradient_norm = objective_value;

  for Nteich = 0:N_teichon_additions
    % Add new teichon location
    if Nteich > 0
      % Stop adding teichons if we're really close
      if obj(teichon_strengths) < 1e-6;
        break;
      end

      objgrad = @(a,b) objective(b, a, theta_0, theta_1, cr_indices, fc, dt);
      [teichon_strengths, teichon_locations] = teichon_addition_avgvalue(teichon_strengths, teichon_locations, objgrad);
      N_teichons = N_teichons + 1;
    end

    % Initialize gradient descent
    cg_state.method = 'gdp';
    cg_state.maxiter = N_gradient_descent_iterations;
    cg_state.objtol = 1e-6;  % Stops if cross-ratio norm smaller than this
    cg_state.updatetol = 1e-12;
    cg_state.verbosity = 2;
    cg_state.increase_period = 5; 
    cg_state = nonlinear_cg_state_initialization(cg_state);
    obj = @(x) objective(teichon_locations, x, theta_0, theta_1, cr_indices, fc, dt);
    cg_state.current_state.objective = obj(teichon_strengths);

    % Compute projection that is used to project gradient
    [garbage,garbage,garbage,gradproj] = horizontal_lift(teichon_locations, teichon_strengths, teichon_locations);

    % All the guts:
    while (cg_state.current_state.iteration_count < cg_state.maxiter) && ...
          not(cg_state.current_state.converged)
      % Compute gradient, projection 
      [L, gradient] = objective(teichon_locations, teichon_strengths, theta_0, theta_1, cr_indices, fc, dt);

      gradient = gradproj(gradient.');
      if norm(gradient) > 1
        gradient = gradient/norm(gradient);
      end

      % Gradient descent
      [teichon_strengths, cg_state] = nonlinear_cg_driver(obj, gradient, teichon_strengths, cg_state);
    end

    % Now do nonlinear cg iterations
    cg_state.method = 'pr';
    cg_state.maxiter = N_cg_iterations;
    cg_state.updatetol = 1e-20;
    cg_state.objtol = 1e-6;
    cg_state.verbosity = 2;
    cg_state.increase_period = 10;
    cg_state = nonlinear_cg_state_initialization(cg_state);
    %---------------------------------------------------------------------------
    % If you comment out the following line, the method will use the Euclidean ip.
    %---------------------------------------------------------------------------
    cg_state.gradient_ip = @(a,b) a'*greens_matrix(teichon_locations)*b;
    cg_state.current_state.objective = obj(teichon_strengths);

    while (cg_state.current_state.iteration_count < cg_state.maxiter) && ...
          not(cg_state.current_state.converged)
      [L, gradient] = objective(teichon_locations, teichon_strengths, theta_0, theta_1, cr_indices, fc, dt);
      gradient = gradproj(gradient.');

      [teichon_strengths, cg_state] = nonlinear_cg_driver(obj, gradient, teichon_strengths, cg_state);
    end
    [L, gradient] = objective(teichon_locations, teichon_strengths, theta_0, theta_1, cr_indices, fc, dt);

  end

end

theta_exterior = map_1.exterior_vertices;

%-----------------------------------------------------------------------
% All done: (teichon_locations, teichon_strengths) contains your answer
% Below is just post-processing to get some pictures
%-----------------------------------------------------------------------

%theta_exterior= map_0.exterior_vertices;
%theta_0 = map_0.interior_vertices;
%[garbage, teichon_locations_1, theta_1] = epdiff_teichon(teichon_strengths, teichon_locations, theta_0, 1, 'dt', dt);
%
%% This does all the magic: (1) computes the weld for the map defined by the
%% current teichon configuration, and (2) applies a Moebius map to the shape
%% that scales+shifts it to match the 'real' shape. The matching procedure can
%% take some time.
%computed_map_1 = GeodesicZipperWeld([], theta_1, theta_exterior);
%computed_map_1 = computed_map_1.match(map_1);
%
%% If for some crazy reason you want the locations of the teichons and
%% cross-ratio points on the shapes, here you go.
%cr_points_computed_map_1 = computed_map_1.map_to_shape(exp(i*theta_1));
%cr_points_exact_map_1 = map_1.map_to_shape(exp(i*theta_1));
%teichon_points_computed_map_1 = computed_map_1.map_to_shape(exp(i*teichon_locations_1));
%teichon_points_exact_map_1 = map_1.map_to_shape(exp(i*teichon_locations_1));
%
%% Just plot
%figure;
%plot([map_1.z; map_1.z(1)], 'r'); hold on;
%plot([computed_map_1.z; computed_map_1.z(1)], 'b'); axis equal;
%legend('Exact', 'Computed');
